
# Proximity Operator Repository 

Proximity operators have become increasingly important tools as basic building blocks of proximal splitting algorithms, a class of algorithms that decompose complex composite convex optimization methods into simple steps involving one of the functions present in the model. This package provides implantations of the proximity operator of various functions (function of scalar variable or multivariate,convex and nonconvex functions, indicator functions...). 

For more information see the [full project](http://proximity-operator.net/) containing technical details, a tutorial and code implantation in matlab.

## Installation

To install from the shell:

    pip install proximity-operator


## How to use

Please visit our [website](http://proximity-operator.net/)  for more tutorial and more details.

To import the package:

    import proximityOperator as prox

## Examples:

Evaluates the function 'AbsValue':

    >>> import proximityOperator as prox
    >>> prox.AbsValue()(-3)
    3

Use a scale factor 'gamma':

    >>> prox.AbsValue(gamma=2)(3)
    6
  
Compute the proximity operator:

    >>> AbsValue().prox( 3)
    2
    >>> AbsValue().prox([ -3., 1., 6.])
    array([-2.,  0.,  5.])


Use a scale factor 'gamma'>0 to commute the proximity operator of gamma*f :

    
    >>> AbsValue(gamma=2).prox([ -3., 1., 6.])
     array([-1.,  0.,  4.])

Or equivalently by using the parameter 'tau':
        
    >>> AbsValue().prox([ -3., 1., 6.], tau=2)
    array([-1.,  0.,  4.])

## An example with matrix variable


     >>> import numpy as np
     >>> from proximityOperator import AffineBariier
     >>>
     >>> x=np.array([1,2,3])
     >>> a= np.array([-1, 5, 3])
     >>> b= 3.5
     >>> AffineBariier(x,b)(x)
     array(inf)      

As the result below is infinite, that means 'x' does not belong to the affine set.

Projection of x onto the affine set:

     >>> px= AffineBariier(x,b).prox(x) 
     >>> px
     >>> AffineBariier(x,b)(px)
     array(-0.17973361)    #

As we could have suspected, the projection of 'x' belongs to the affine set.
     
     >>> x=np.arange(6)
     >>> x=x.reshape((2,3))
     >>> x
     array([[0, 1, 2],
           [3, 4, 5]])
     >>> a =np.ones
     >>> a[0,:]=2
     >>> a
     array([[2., 2., 2.],
            [1., 1., 1.]])
     >>> b=np.array([-1, 2, 4])
     
     Set 'direction=0' to process along the rows of the matrix 'x':
         
     >>> AffineBariier(x,b, direction=0)(x)
     array([inf, inf, inf])
     
     Projection of x onton the affine set:
         
     >>> px = AffineBariier(x,b, direction=0).prox(x)  
     >>> px
     array([[0.        , 0.05537521, 0.19926499],
            [0.        , 0.22150085, 0.49816248]])
     >>> AffineBariier(x,b, direction=0)(px)
     array([-0.        , -0.05696748, -0.10495226])
