#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
 Version : 1.0 (06-10-2022)
 
 Author  : Mbaye DIONGUE

 Copyright (C) 2022
 
 This file is part of the codes provided at http://proximity-operator.net
 
 By downloading and/or using any of these files, you implicitly agree to 
 all the terms of the license CeCill-B (available online).
"""

import numpy as np

class BoxConstraint :
    """
    This class computes the projection onto the constraint set:

                           low <= x <= high
                           
     INPUTS
    ========
     x    - ND array
     low  - scalar or ND array with the same size as 'x'
     high - scalar or ND array with the same size as 'x'
    """
    
    def __init__(self, low, high):
        if np.any(low > high):
            raise Exception("'low' must be lower than 'high'")
        self.low=low
        self.high=high
        
        
    # proximal operator (i.e projetion on the box)   
    def prox(self, x):
        self._check(x)
        return np.maximum(self.low, np.minimum(x, self.high))
    
    # indicator of the box
    def __call__(self, x):
        self._check(x)
        if np.all(self.low <= x) and np.all(x <= self.high):
            return 0
        else:
            return np.inf
    
    def _check(self, x):
        if ( np.size(self.low)>1) and (np.shape(self.low) != np.shape(x)):
            raise Exception("'low' must be either scalar or the same size as 'x'")

        if (np.size(self.high)>1) and (np.shape(self.high) != np.shape(x)):
            raise Exception("'high' must be either scalar or the same size as 'x'")
