#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
 Version : 1.0 (06-10-2022)
 
 Author  : Mbaye DIONGUE

 Copyright (C) 2022
 
 This file is part of the codes provided at http://proximity-operator.net
 
 By downloading and/or using any of these files, you implicitly agree to 
 all the terms of the license CeCill-B (available online).
"""

import numpy as np

class Hyperplane:
    """
    This class computes the projection onto the hyperplane set:

                           u.T * x = mu
                           
     where u.T denotes the transpose of u
     

     INPUTS
    ========
     x    - ND array
     u    - ND array with the same size as 'x'
     mu    - scalar
    """
    
    def __init__(self, u, mu, direction=0):
        self.u=u
        self.mu=mu
        self.direc=direction
        
    # proximal operator (i.e projetion on the hyperplane)   
    def prox(self, x):
        self._check(x)
        coef=(self.mu - np.sum(x * self.u)) / (np.linalg.norm(self.u) ** 2) 
        return x + coef* self.u
    
    # indicator of the hyperplane
    def __call__(self, x):
        self._check(x)
        tol=1e-10
        if abs( np.sum(x * self.u) - self.mu) <=tol:
            return 0
        else:
            return np.inf
    
    def _check(self, x):
        if  (np.shape(self.u)) != np.shape(x):
            raise Exception("'u' must have the same size as 'x'")

