#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
 Version : 1.0 (06-14-2022)
 
 DEPENDENCIES:
     - 'Max.py' in the folder 'multi'
     - 'L2Norm.py' in the folder 'multi'
     
 Author  : Mbaye DIONGUE

 Copyright (C) 2022
 
 This file is part of the codes provided at http://proximity-operator.net
 
 By downloading and/or using any of these files, you implicitly agree to 
 all the terms of the license CeCill-B (available online).
"""
import numpy as np
from prox.multi.L2Norm import L2Norm

class L2Ball:
    """
    Computes the projection onto the constraint set:
    
                        ||x||_2 <= eta

     INPUTS
    ========
     x         - ND array
     eta       - positive scalar
     direction - integer, direction of block-wise processing
    """
    
    def __init__(self, eta, direction=None):
        if np.size(eta)>1 or eta<=0:
            raise Exception("'eta' must be a positive scalar")
        self.eta=eta
        self.gamma=1
        self.direction=direction

    def prox(self, x):
        return  x - L2Norm(gamma=1, direction=self.direction).prox(x, self.eta)
    
    def __call__(self, x):
        TOL=1e-10
        l2_x= np.linalg.norm(x, ord=2, axis=self.direction, keepdims=True)
        if np.all(l2_x <= self.eta+ TOL):
            return 0
        return np.inf
    