#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
 Version : 1.0 (06-14-2022)
  
 Author  : Mbaye DIONGUE

 Copyright (C) 2022
 
 This file is part of the codes provided at http://proximity-operator.net
 
 By downloading and/or using any of these files, you implicitly agree to 
 all the terms of the license CeCill-B (available online).
"""

import numpy as np

class LinfBall:
    """
    Computes the projection onto the constraint set:
    
                        ||x||_inf <= eta
                        
        where ||x||_inf = max{ |x1|, ..., |xn|} is the infinity norm

     INPUTS
    ========
     x         - ND array
     eta       - positive scalar
     direction - integer, direction of block-wise processing
    """
    
    def __init__(self, eta, direction=None):
        if np.size(eta)>1 or eta<=0:
            raise Exception("'eta' must be a positive scalar")
        self.eta=eta
        self.gamma=1
        self.direction=direction

    def prox(self, x):
        p=np.sign(x)*np.minimum( np.abs(x), self.eta)
        return p
    
    def __call__(self, x):
        TOL=1e-10
        linf_x= np.max( np.abs(x), axis=self.direction, keepdims=True)
        if np.all( linf_x <= self.eta +TOL) :
            return 0
        return np.inf
    