#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
 Version : 1.0 (06-8-2022)
 DEPENDENCIES:
     'pava_python.py' in the folder 'utils'
     
 Author  : Mbaye DIONGUE

 Copyright (C) 2022
 
 This file is part of the codes provided at http://proximity-operator.net
 
 By downloading and/or using any of these files, you implicitly agree to 
 all the terms of the license CeCill-B (available online).
"""

import numpy as np
from prox.utils.pava_python import pava_python

class MonotoneCone:
    """
    This class computes the projection onto the monotone cone defined as follows:

                               x1 <= x2 <= ... <= xN
                             
                           
     where (1, ..., 1) is a ND array with all components equal to one, and (1,..., 1).T denotes its transpose
      
     When the input 'x' is an array, the computation can vary as follows:
         - dir = 0 --> 'x' is processed as a single vector [DEFAULT]
         - dir > 0 --> 'x' is processed block-wise along the specified direction
     
     INPUTS
    ========
     x    - ND array
     eta  - positive, scalar or ND array compatible with the blocks of 'x'
     dir  - integer, direction of block-wise processing
     """
    
    def __init__(self):
        self.direc=0
        
    # proximal operator (i.e projetion on the monotone cone)   
    def prox(self, x):
        
        p= pava_python(x)
        return p
    
    # indicator of the monotone cone
    def __call__(self, x):
        if np.size(x) <= 1:
            return 0
        if np.all( x[:-1] <= x[1:]):
            return 0
        return np.inf
  

