#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
 Version : 1.0 (06-13-2022)
 
 DEPENDENCIES: 
     - 'AbsValue.py' in the folder 'scalar'
     - 'prox_svd.py' in the folder 'utils'
     - 'fun_svd.py' in the folder 'utils'     
     
 Author  : Mbaye DIONGUE

 Copyright (C) 2022
 
 This file is part of the codes provided at http://proximity-operator.net
 
 By downloading and/or using any of these files, you implicitly agree to 
 all the terms of the license CeCill-B (available online).
"""

import numpy as np 
from prox.scalar.AbsValue import AbsValue
from prox.utils.prox_svd import prox_svd
from prox.utils.fun_svd import fun_svd

class NuclearNorm:
    """
    Computes the proximity operator of (gamma*tau)*f where f is the nuclear  as :
    
                               
                        f(x) = ||X||_N = ||s||_1
                             
            where X = U*diag(s)*V.T \in R^{M*N} 
     INPUTS
    ========
     x         - (M,N) -array_like ( representinf a M*N matrix )
     gamma     - positive, scalar or ND array compatible with the size of 'x'
     """
     
    def __init__(self, gamma=1):
        if np.any( gamma <= 0 ):
            raise Exception("'gamma' (or all of its components if it is an array) must be strictly positive")
        self.gamma = gamma
    
        
    def prox(self, x, tau=1):
        self._check( x, tau)
        scale= tau*self.gamma
        p=prox_svd(x, 1, AbsValue(gamma=scale).prox )
        return p
    
    def __call__(self, x):
        return fun_svd(x, 1, AbsValue(gamma= self.gamma) )
    
    def _check(self, x, tau, name_name="'tau'"):
       if np.any( tau <= 0 ):
           raise Exception(name_name+" (or all of its components if it is an array) must be strictly positive")
       
       if ( np.size(tau) > 1)  and (np.shape(tau) != np.shape(x) ):
           raise Exception(name_name+" must be either scalar or the same size as 'x'")
