#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
 (27-04-2017) (updated: 07-06-2022)
 
 Author  : Giovanni Chierchia (update: Mbaye Diongue)

 Copyright (C) 2022
 
 DEPENDENCIES:
     
 This file is part of the codes provided at http://proximity-operator.net
 
 By downloading and/or using any of these files, you implicitly agree to 
 all the terms of the license CeCill-B (available online).
 """
   

from .Thresholder import Thresholder

class AbsValue(Thresholder):
    """
    This class computes the proximity operator of the function

                       f(x) = gamma *tau * |x|

    'gamma * tau' is the scale factor
    
    When the input 'x' is an array, the output is computed element-wise.

     INPUTS
    ========
    x     - scalar or ND array
    gamma - positive, scalar or ND array with the same size as 'x' [default: gamma=1]
    tau   - positive, scalar or ND array with the same size as 'x' [default: tau=1] [only for the method 'prox']  
    
    ========
    Examples
    ========
    
    >>> AbsValue()(-3)
    3
    >>> AbsValue(gamma=2)(3)
    6
    
    >>> AbsValue().prox( 3)
    2
    >>> AbsValue().prox([ -3., 1., 6.])
    array([-2.,  0.,  5.])
    
   Use a scale factor 'gamma'>0 to compute the proximity operator of  the function 'gamma*f'
    
    >>> AbsValue(gamma=2).prox([ -3., 1., 6.])
     array([-1.,  0.,  4.])
     
    This equivalent to use the parameter 'tau' of the method 'prox' to compute the 
    proximity operator of  the function 'tau*f'
        
    >>> AbsValue().prox([ -3., 1., 6.], tau=2)
    array([-1.,  0.,  4.])
    """
    
    def __init__(self, gamma=1):

        super().__init__( -1, 1, gamma)