#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
 Version : 1.0 (06-09-2022)
 
 Author  : Mbaye DIONGUE

 Copyright (C) 2022
 
 This file is part of the codes provided at http://proximity-operator.net
 
 By downloading and/or using any of these files, you implicitly agree to 
 all the terms of the license CeCill-B (available online).
"""

import numpy as np

class ArgsinhActi:
    """
    This class computes the proximity operator of the inverse hyperbolic sine activation function
    
                    f(x) = cosh(x) -(x^2)/2

    When the input 'x' is an array, the output is computed element-wise.


     INPUTS
    ========
     x     - scalar or ND array
     
    ========
    Examples
    ========
    
    Evaluate the 'direct' function :
    >>> ArgsinhActi()(-0.5)
     1.0026259652063807
    >>>ArgsinhActi()([-2., -0.5, 0, 0.25, 3])
    array([1.76219569, 1.00262597, 1.        , 1.0001631 , 5.567662  ])
    
    Compute the proximity operator at a given point :
    >>> ArgsinhActi().prox(3)
     1.8184464592320668s
    >>> ArgsinhActi().prox([ -3., 0.5, 6.])
    array([-1.81844646,  0.48121183,  2.49177985])
    """
    def __init__(self):
        self.gamma=1

    def prox(self, x):
        return np.arcsinh(x)
        
    def __call__(self, x):
        if not isinstance(x, np.ndarray):
            x= np.array(x)
        return np.cosh(x) - 0.5 * x ** 2

      
        