#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
 Version : 1.0 (27-04-2017)
 
 DEPENDENCIES:
     'Threshholder.py' in the folder scalar
     
 Author  : Giovanni Chierchia  (updated: 07-06-2022)

 Copyright (C) 2017
 
 This file is part of the codes provided at http://proximity-operator.net
 
 By downloading and/or using any of these files, you implicitly agree to 
 all the terms of the license CeCill-B (available online).
 """

from prox import Thresholder

class HingeLoss(Thresholder) :
    """
    This class computes the proximity operator of the function

                      f(x) = gamma * max{x,0}

    When the input 'x' is an array, the output is computed element-wise.

     INPUTS
    ========
     x     - ND array
     gamma  - positive, scalar or array with the same size as 'x' [default value: gamma=1]
     tau    - positive, scalar or array with the same size as 'x' [default: tau=1] [Only for the method 'prox']
     
    =======
    Examples
    ========
 
     Evaluate the 'direct' function :
    
     >>> HingeLoss()( 0.5 )
     0.5
     
      Compute the result element-wise for vector inputs :
          
     >>> HingeLoss(gamma=2)( [-1, 6, 0.25] )  
     array([0. , 6. , 0.5])
     
     Compute the proximity operator at a given point :
         
     >>> HingeLoss().prox(  [-2, 3, 4 ])
     array([-2,  2,  3])
     
     Use a scale factor 'gamma'>0 to compute the proximity operator of  the function 'gamma*f'
    
     >>> HingeLoss(gamma=2.5).prox( [-2, 3, 4, np.e ] )
     array([-2.        ,  0.5       ,  1.5       ,  0.21828183])
     
      This equivalent to use the parameter 'tau' of the method 'prox' to compute the 
      proximity operator of  the function 'tau*f'
      
     >>> HingeLoss().prox( [-2, 3, 4, np.e ] , tau=2.5 )
     array([-2.        ,  0.5       ,  1.5       ,  0.21828183])
    """
        
    def __init__(self, gamma=1):

        super().__init__( 0, 1, gamma)