#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
 Original Version : 1.0 (01-02-2019), updated  (Thu Jun  9 2022)
 
 Author  : MOHAMED KERROUMI

 Copyright (C) 2019
 
 This file is part of the codes provided at http://proximity-operator.net
 
 By downloading and/or using any of these files, you implicitly agree to 
 all the terms of the license CeCill-B (available online).

"""

import numpy as np

class LogActi:
    """
    This class computes the proximity operator of the logarithm activation function
    
            f(x) = cosh(x)-(x^2)/2
    
     When the input 'x' is an array, the output is computed element-wise.
    
     INPUTS
    ========
     x     - scalar or ND array
     
     =======
     Examples
     ========
     
     >>> LogActi()( -3 )
      11.585536923187668
     >>> LogActi()( [1, -1.5, -0.5] )
     array([0.21828183, 0.85668907, 0.02372127])
   
     Compute the proximity operator at a given point :
    
     >>> LogActi().prox(-3)
     -1.3862943611198906
     >>> LogActi().prox(  [-2, 3, -0.5] )
     array([-1.09861229,  1.38629436, -0.40546511])
    """
    def __init__(self):
        self.gamma=1

    def prox(self, x):
        return  np.sign(x) * np.log(1 + np.abs(x)) 
        
    def __call__(self, x):
        if np.size(x)>1 and (not isinstance(x, np.ndarray)):
            x=np.array(x)
        return np.exp(np.abs(x)) - np.abs(x) - 1 - 0.5 * x ** 2

