# -*- coding: utf-8 -*-
"""
Created 14-06-2022
 Version : 1.0 (06-14-2022)
 
 Author  : Mbaye Diongue
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 Copyright (C) 2022

 This file is part of the codes provided at http://proximity-operator.net

 By downloading and/or using any of these files, you implicitly agree to 
 all the terms of the license CeCill-B (available online).
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
"""
import numpy as np


def prox_svd( x, gamma, prox_phi, hermitian=False):
    """
    This function computes the proximity operator of the matrix function:
    
                        f(X) = gamma * phi(s)     with   X = U' * diag(s) * V
     
     It is assumed that the matrices are stored along the dimensions 0 and 1.
    
      INPUTS
     ========
      x       - ND array
      gamma   - positive, scalar or ND array compatible with the size of 'x'
     prox_phi - function handle with two arguments at least
    """
   
    # spectral decomposition
    """
    try : # try use cpp code singular value decompotion first  (it's faster)
        if np.size(x)[1] > np.size(x)[0]:
            u,s,v = sv_dec_fat(x)
        else:
            u,s,v = sv_dec(x)
    except Exception as e:
        
        u, s, vh = np.linalg.svd(x, full_matrices=True)
    """
    
    u, s, vh = np.linalg.svd(x, full_matrices=True, hermitian=hermitian)
    # prox computation
    g = prox_phi(s, gamma)
    
    gmat = np.zeros_like(1.0*x)
    diag_g=np.diag(g)
    gmat[:np.shape(diag_g)[0], :np.shape(diag_g)[1]] = diag_g
    # spectral reconstruction
    #y = sv_rec(u, g, v)
    y= np.dot(u, np.dot(gmat, vh))
    return y